local RM=LibStub("AceAddon-3.0"):GetAddon("Raid Mover");

--++++++++++++++++++++++++++   SLASH COMMANDS    ++++++++++++++++++++++++++++
function RM:initializeCommands()

    local commands = {}
    function commands.test()
        self:Print("test")
    end

    function commands.shorthelp()
        self:Print("  USAGE  ")
        self:Print("/raidmover or /rm [command]")
        self:Print("gui, inv, invlist, swap, layout, shiftdown, shiftup")
        self:Print("Type  /raidmover help  for a description of each command")
        self:Print("Type  /raidmover gui  for the GUI window")
        self:Print("Type  /raidmover config  to set some options")
        self:Print("Type  /raidmover_options  to change configuration options from a macro")
    end

    function commands.more()
        self:Print("================HELP================")
        self:Print("/rm [command]")
        self:Print("gui: Open the command window")
		self:Print("config: Open the configuration window")
        self:Print("invlist: Invite the players from a saved invite list OR saved raid layout.")
		self:Print("   USAGE: /rm invlist LISTNAME")
		self:Print("saveinvlist: Save the current raid members to an invite list.")
		self:Print("   USAGE: /rm saveinvlist LISTNAME")
		self:Print("showinvlist: Shows the current players on the specified saved invite list OR saved raid layout.")
		self:Print("   USAGE: /rm showinvlist LISTNAME")
		self:Print("save: Saves a raid layout")
		self:Print("   USAGE: /rm save LAYOUT_NAME")
		self:Print("restore: Restores a raid layout")
		self:Print("   USAGE: /rm restore LAYOUT_NAME")
		self:Print("remove: Removes a raid layout")
		self:Print("   USAGE: /rm remove LAYOUT_NAME")
		self:Print("create: Creates a new raid layout")
		self:Print("   USAGE: /rm create LAYOUT_NAME")
		self:Print("rename: Renames a raid layout")
		self:Print("   USAGE: /rm rename SAVED_LAYOUT_NAME NEW_LAYOUT_NAME")
		self:Print("copy: Copies a saved raid layout")
		self:Print("   USAGE: /rm copy SAVED_LAYOUT_NAME NEW_LAYOUT_NAME")
		self:Print("listsave: Lists your saved raid layouts, or add a layout name to list the players")
		self:Print("   USAGE: /rm listsave [LAYOUT_NAME]")
		self:Print("addplayer: Add a player to, or change a players group in a raid layout, case-sensitive")
		self:Print("   USAGE: /rm addplayer LAYOUTNAME PLAYER GROUP#")
		self:Print("removeplayer: Removes a player from a raid layout, case-sensitive");
		self:Print("   USAGE: /rm removeplayer LAYOUTNAME PLAYER");
    end
	
    function commands.help()
        commands.more()
    end

    function commands.gui()
		RM:ToggleMover()
    end

    function commands.config()
        self:showConfigWindow()
    end
	
    function commands.restore(savedlayout)
		if savedlayout then
			self:Print("Setting raid layout to: " ..savedlayout)
			self:startMover(savedlayout)
		end
    end
	
	function commands.save(savename)
		self:Print("Saving raid layout as: " ..savename)
		self:saveRaid(savename)
	end
	
	function commands.listsave(layout)
		if layout then
			self:printLayout(layout)
		else	
			self:Print("Listing Saved Raid Layouts:")
			for k, v in pairs(RM_Vars.SavedRaidLayouts) do
				self:Print(k)
			end
		end	
	end
	    
	function commands.create(layoutname)
		if layoutname then
			self:createLayout(layoutname);
		else
			self:Print("I require a layout name please.")
		end
	end
	
	function commands.copy(copyname, newname)
		if newname and copyname then
			self:copyLayout(copyname, newname)
		else
			self:Print("I require proper layout names please.")
		end
	end
	
	function commands.remove(layoutname)
		if layoutname then
			self:removeLayout(layoutname);
		else
			self:Print("I require a layout name please.")
		end
	end
	
	function commands.addplayer(layoutname, player, subgrp)
		if layoutname and player and subgrp then
			self:addPlayer(layoutname, player, subgrp)
		else 
			self:Print("Invalid arguments.")
		end
	end
	
	function commands.removeplayer(layoutname, playername)
		if layoutname and playername then
			self:removePlayer(layoutname, playername);
		else
			self:Print("Invalid arguments.")
		end
	end
	
    function commands.swap(groupAstr, groupBstr)
        local groupA = tonumber(groupAstr)
        local groupB = tonumber(groupBstr)

        if groupA and groupB then
            self:Print("Swapping groups " .. tostring(groupA) .. " and " 
                   .. tostring(groupB))
            self:patch(self:swapGroup(groupA, groupB))
        else
            self:Print("Error, type two group numbers")
        end
    end

    function commands.convert()
        self:Print("Converting party to a raid group")
        self:convert()
    end

    function commands.invite(...)
        self:Print("Inviting " .. strjoin(", ", ...))
        self:invite(...)
    end

	function commands.getcurrentraid()
		self:getCurrentRaid()
	end
	
	function commands.runattendance(event, boss)
		self:runAttendance(event, boss)
	end
	
	function commands.attendance()
		self:runAttendanceTarget()
	end
	
	function commands.fixattendance()
		RM:Print("Fixing attendance records...")
		self:fixAttendance()
	end
	
	function commands.attendancetarget()
		self:runAttendanceTarget()
	end
	
	function commands.fixlayouts()
		RM:Print("Fixing saved raid layouts")
		self:fixLayouts()
	end
	
	function commands.clearattendance(event, arg)
		self:clearAttendance(event, arg)
	end
	--arg: "all_dates" "all_dates_boss" "all_dates_boss_diff" "all_kill" "all_kill_boss" "all_kill_boss_diff"
	function commands.killcount(arg, playername, boss, difficulty)
		self:getKillCount(arg, playername, boss, difficulty)
	end
    commands.inv = commands.invite

	function commands.getraidspecs()
		self:getRaidSpecs()
	end
	
	
    self.commands = commands
	collectgarbage()
end

function RM:showConfigWindow()
    LibStub("AceConfigDialog-3.0"):Open("Raid Mover.Profiles")
end

function RM:onSlashCommand(str)
    local args = {self:GetArgs(str, 100)}
    local command = table.remove(args, 1)
	
    if not command then
        self.commands.shorthelp()
    elseif self.commands[command] then
        local fn = self.commands[command]
        fn(unpack(args))
    else
        self:Print("No such command '" .. command .. "'")
    end
end
